"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeView = void 0;
const documenter_ui_heading_1 = require("@teambit/documenter.ui.heading");
const classnames_1 = __importDefault(require("classnames"));
const react_1 = __importStar(require("react"));
const documenter_ui_code_snippet_1 = require("@teambit/documenter.ui.code-snippet");
const react_syntax_highlighter_1 = require("react-syntax-highlighter");
const code_ui_queries_get_file_content_1 = require("@teambit/code.ui.queries.get-file-content");
const prism_light_1 = __importDefault(require("react-syntax-highlighter/dist/esm/prism-light"));
const markdown_1 = __importDefault(require("react-syntax-highlighter/dist/esm/languages/prism/markdown"));
const base_ui_constants_storage_1 = require("@teambit/base-ui.constants.storage");
const base_react_navigation_link_1 = require("@teambit/base-react.navigation.link");
const code_view_module_scss_1 = __importDefault(require("./code-view.module.scss"));
prism_light_1.default.registerLanguage('md', markdown_1.default);
const extractLineNumber = (hash) => {
    if (!hash)
        return null;
    const match = hash.match(/#l(\d+)/);
    return match ? parseInt(match[1], 10) : null;
};
const extractSearchKeyword = (hash) => {
    if (!hash)
        return null;
    const match = hash.match(/#search=(.*)/);
    return match ? decodeURIComponent(match[1]) : null;
};
function CodeView({ className, componentId, currentFile, icon, currentFileContent, codeSnippetClassName, loading: loadingFromProps, }) {
    const { fileContent: downloadedFileContent, loading: loadingFileContent } = (0, code_ui_queries_get_file_content_1.useFileContent)(componentId, currentFile, !!currentFileContent);
    const loading = loadingFromProps || loadingFileContent;
    const location = (0, base_react_navigation_link_1.useLocation)();
    const navigate = (0, base_react_navigation_link_1.useNavigate)();
    const [scrollBlock, setScrollBlock] = react_1.default.useState('center');
    const highlightedLineRef = react_1.default.useRef(null);
    const fileContent = currentFileContent || downloadedFileContent;
    const title = (0, react_1.useMemo)(() => currentFile === null || currentFile === void 0 ? void 0 : currentFile.split('/').pop(), [currentFile]);
    const lang = (0, react_1.useMemo)(() => {
        const langFromFileEnding = currentFile === null || currentFile === void 0 ? void 0 : currentFile.split('.').pop();
        // for some reason, SyntaxHighlighter doesnt support scss or sass highlighting, only css. I need to check how to fix this properly
        if (langFromFileEnding === 'scss' || langFromFileEnding === 'sass')
            return 'css';
        if (langFromFileEnding === 'mdx')
            return 'md';
        return langFromFileEnding;
    }, [fileContent]);
    const lineNumber = extractLineNumber(location === null || location === void 0 ? void 0 : location.hash);
    const searchKeyword = extractSearchKeyword(location === null || location === void 0 ? void 0 : location.hash);
    react_1.default.useEffect(() => {
        if ((highlightedLineRef === null || highlightedLineRef === void 0 ? void 0 : highlightedLineRef.current) && (lineNumber || searchKeyword)) {
            highlightedLineRef === null || highlightedLineRef === void 0 ? void 0 : highlightedLineRef.current.scrollIntoView({ behavior: 'smooth', block: scrollBlock });
        }
    }, [lineNumber, searchKeyword, highlightedLineRef === null || highlightedLineRef === void 0 ? void 0 : highlightedLineRef.current, scrollBlock]);
    if (!fileContent && !loading && currentFile)
        return react_1.default.createElement(EmptyCodeView, null);
    const lineNumberStyle = react_1.default.useCallback((_lineNumber) => {
        return {
            onClick: () => {
                setScrollBlock('nearest');
                navigate(`${location === null || location === void 0 ? void 0 : location.pathname}#l${_lineNumber}`);
            },
            style: { cursor: 'pointer' }
        };
    }, [location === null || location === void 0 ? void 0 : location.pathname]);
    const customRenderer = react_1.default.useCallback(({ rows, stylesheet, useInlineStyles }) => {
        let isKeywordHighlighted = false;
        return rows.map((node, index) => {
            const lineText = node.children
                .map((child) => { var _a, _b, _c; return (_c = (_b = (_a = child.children) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) !== null && _c !== void 0 ? _c : ''; })
                .join('')
                .trim();
            const lineNum = index + 1;
            const matchesSearchWord = !isKeywordHighlighted && searchKeyword && lineText.includes(searchKeyword);
            const isHighlighted = lineNum === lineNumber || matchesSearchWord;
            if (isHighlighted)
                isKeywordHighlighted = true; // Ensure only the first match is highlighted
            return (react_1.default.createElement("div", { ref: isHighlighted ? highlightedLineRef : null, key: index, className: (0, classnames_1.default)(isHighlighted && code_view_module_scss_1.default.customRow) }, (0, react_syntax_highlighter_1.createElement)({
                key: index,
                node,
                stylesheet,
                useInlineStyles,
            })));
        });
    }, [lineNumber]);
    return (react_1.default.createElement("div", { className: (0, classnames_1.default)(code_view_module_scss_1.default.codeView, className) },
        react_1.default.createElement(documenter_ui_heading_1.H1, { size: "sm", className: code_view_module_scss_1.default.fileName },
            currentFile && react_1.default.createElement("img", { className: code_view_module_scss_1.default.img, src: icon }),
            react_1.default.createElement("span", null, title)),
        react_1.default.createElement(documenter_ui_code_snippet_1.CodeSnippet, { className: code_view_module_scss_1.default.codeSnippetWrapper, frameClass: (0, classnames_1.default)(code_view_module_scss_1.default.codeSnippet, codeSnippetClassName), showLineNumbers: true, language: lang, renderer: customRenderer, lineProps: lineNumberStyle, showInlineLineNumbers: true }, fileContent || '')));
}
exports.CodeView = CodeView;
function EmptyCodeView() {
    return (react_1.default.createElement("div", { className: code_view_module_scss_1.default.emptyCodeView },
        react_1.default.createElement("img", { src: `${base_ui_constants_storage_1.staticStorageUrl}/harmony/empty-code-view.svg` }),
        react_1.default.createElement("div", null, "Nothing to show")));
}
//# sourceMappingURL=code-view.js.map